@@ -1,13 +1,12 @@
 ;;; pulsar.el --- Pulse highlight on demand or after select functions -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022-2023  Free Software Foundation, Inc.
+;; Copyright (C) 2022-2024  Free Software Foundation, Inc.
 
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
-;; Maintainer: Pulsar Development <~protesilaos/pulsar@lists.sr.ht>
-;; URL: https://git.sr.ht/~protesilaos/pulsar
-;; Mailing-List: https://lists.sr.ht/~protesilaos/pulsar
-;; Version: 1.0.1
-;; Package-Requires: ((emacs "27.1"))
+;; Maintainer: Protesilaos Stavrou <info@protesilaos.com>
+;; URL: https://github.com/protesilaos/pulsar
+;; Version: 1.1.0
+;; Package-Requires: ((emacs "28.1"))
 ;; Keywords: convenience, pulse, highlight
 
 ;; This file is NOT part of GNU Emacs.
@@ -33,6 +32,14 @@
 ;; takes place when either `pulsar-mode' (buffer-local) or
 ;; `pulsar-global-mode' is enabled.
 ;;
+;; By default, Pulsar does not try behave the same way for a
+;; function's aliases.  If those are not added explicitly to the
+;; `pulsar-pulse-functions', they will not have a pulse effect.
+;; However, the user option `pulsar-resolve-pulse-function-aliases'
+;; can be set to a non-nil value to change this behaviour, meaning
+;; that Pulsar will cover a function's aliases even if those are not
+;; explicitly added to the `pulsar-pulse-functions'.
+;;
 ;; The overall duration of the highlight is determined by a combination
 ;; of `pulsar-delay' and `pulsar-iterations'.  The latter determines the
 ;; number of blinks in a pulse, while the former sets their delay in
@@ -72,44 +79,73 @@ Extension of `pulse.el'."
 ;;;; User options
 
 (defcustom pulsar-pulse-functions
-  '(recenter-top-bottom
-    move-to-window-line-top-bottom
-    reposition-window
+  '(backward-page
     bookmark-jump
-    other-window
-    delete-window
     delete-other-windows
+    delete-window
+    evil-goto-first-line
+    evil-goto-line
+    evil-scroll-down
+    evil-scroll-line-to-bottom
+    evil-scroll-line-to-center
+    evil-scroll-line-to-top
+    evil-scroll-page-down
+    evil-scroll-page-up
+    evil-scroll-up
     forward-page
-    backward-page
-    scroll-up-command
-    scroll-down-command
+    goto-line
+    handle-switch-frame
+    logos-backward-page-dwim
+    logos-forward-page-dwim
+    handle-select-window
+    move-to-window-line-top-bottom
+    narrow-to-defun
+    narrow-to-page
+    narrow-to-region
     next-buffer
-    previous-buffer
-    windmove-right
-    windmove-left
-    windmove-up
-    windmove-down
-    windmove-swap-states-right
-    windmove-swap-states-left
-    windmove-swap-states-up
-    windmove-swap-states-down
-    tab-new
-    tab-close
-    tab-next
+    next-multiframe-window
+    org-backward-heading-same-level
+    org-forward-heading-same-level
     org-next-visible-heading
     org-previous-visible-heading
-    org-forward-heading-same-level
-    org-backward-heading-same-level
+    other-window
     outline-backward-same-level
     outline-forward-same-level
     outline-next-visible-heading
     outline-previous-visible-heading
-    outline-up-heading)
-  "Functions that `pulsar-pulse-line' after invocation.
+    outline-up-heading
+    previous-buffer
+    recenter-top-bottom
+    reposition-window
+    scroll-down-command
+    scroll-up-command
+    tab-close
+    tab-new
+    tab-next
+    tab-previous
+    widen
+    windmove-down
+    windmove-left
+    windmove-right
+    windmove-swap-states-down
+    windmove-swap-states-left
+    windmove-swap-states-right
+    windmove-swap-states-up
+    windmove-up)
+  "Functions that call `pulsar-pulse-line' after invocation.
 This only takes effect when `pulsar-mode' (buffer-local) or
 `pulsar-global-mode' is enabled."
   :type '(repeat function)
-  :package-version '(pulsar . "1.0.0")
+  :package-version '(pulsar . "1.1.0")
+  :group 'pulsar)
+
+(defcustom pulsar-resolve-pulse-function-aliases t
+  "When non-nil, resolve function aliases in `pulsar-pulse-functions'.
+This allows pulsar to respect, e.g., `tab-new' \"parent,\"
+`tab-bar-new-tab', and vice-versa, enabling Pulsar to respect
+`tab-bar-new-tab' alias `tab-new'."
+  :type 'boolean
+  :package-version '(pulsar . "1.1.0")
   :group 'pulsar)
 
 (make-obsolete 'pulsar-pulse-on-window-change nil "0.5.0")
@@ -310,6 +346,32 @@ pulse effect."
   (interactive)
   (pulsar--pulse))
 
+;;;###autoload
+(defun pulsar-pulse-region ()
+  "Temporarily highlight the active region if any."
+  (interactive)
+  (when (region-active-p)
+    (let ((beg (region-beginning))
+          (end (region-end)))
+      ;; FIXME 2024-08-29: Finding the lines and columns therein
+      ;; does not work because consecutive pulses cancel each
+      ;; other out, leaving only the last one active.
+      ;;
+      ;; (let* ((columns (rectangle--pos-cols beg end))
+      ;;        (begcol (car columns))
+      ;;        (endcol (cdr columns)))
+      ;;    (lines (list
+      ;;            (line-number-at-pos beg)
+      ;;            (line-number-at-pos end))))
+      ;; (dolist (line lines)
+      ;;   (save-excursion
+      ;;     (goto-char (point-min))
+      ;;     (forward-line (1- line))
+      ;;     (setq beg (progn (move-to-column begcol) (point))
+      ;;           end (progn (move-to-column endcol) (point))))
+      ;;   (pulsar--pulse nil nil beg end)))
+      (pulsar--pulse nil nil beg end))))
+
 ;;;###autoload
 (defun pulsar-highlight-line ()
   "Temporarily highlight the current line.
@@ -410,7 +472,10 @@ For lines, do the same as `pulsar-highlight-line'."
 This is a buffer-local mode.  Also check `pulsar-global-mode'."
   :global nil
   (if pulsar-mode
-      (add-hook 'post-command-hook #'pulsar--post-command-pulse nil 'local)
+      (progn
+        (when pulsar-resolve-pulse-function-aliases
+          (pulsar--resolve-function-aliases))
+        (add-hook 'post-command-hook #'pulsar--post-command-pulse nil 'local))
     (remove-hook 'post-command-hook #'pulsar--post-command-pulse 'local)))
 
 (defun pulsar--on ()
@@ -431,6 +496,40 @@ This is a buffer-local mode.  Also check `pulsar-global-mode'."
 
 (make-obsolete 'pulsar-setup nil "0.3.0")
 
+;; Lifted from Emacs 30 to allow pulsar to remain backward compatbile
+;; with Emacs earlier than Emacs 29.1. TODO: Deprecate this at some
+;; point to prefer Emacs core.
+(defun pulsar--function-alias-p (func &optional _noerror)
+  "Return nil if FUNC is not a function alias.
+If FUNC is a function alias, return the function alias chain."
+  (declare (advertised-calling-convention (func) "30.1")
+           (side-effect-free error-free))
+  (let ((chain nil))
+    (while (and (symbolp func)
+                (setq func (symbol-function func))
+                (symbolp func))
+      (push func chain))
+    (nreverse chain)))
+
+;; This is essentially the inverse of function-alias-p for a list of
+;; function symbols.
+(defun pulsar--find-fn-aliases (fns)
+  "Return a list of aliases for the FNS symbols."
+  (let ((aliases))
+    (mapatoms (lambda (sym)
+                (when (and
+                       (commandp sym)
+                       (memq (symbol-function sym) fns))
+                  (push sym aliases))))
+    aliases))
+
+(defun pulsar--resolve-function-aliases ()
+  "Amend `pulsar-pulse-functions' to respect function aliases."
+  (setq pulsar-pulse-functions
+        (seq-union pulsar-pulse-functions
+                   (seq-union (pulsar--find-fn-aliases pulsar-pulse-functions)
+                              (flatten-list (mapcar (lambda (x) (pulsar--function-alias-p x)) pulsar-pulse-functions))))))
+
 ;;;; Recentering commands
 
 (defun pulsar-recenter-top ()
